@@ -4,13 +4,15 @@ import nltk
 
 
 
+
+
 Pyphen = None
 
 # lang specific constants
 PATH_LANG_DATA=os.path.join(PATH_DATA,'en')
 CMU_DICT_FN=os.path.join(PATH_LANG_DATA,'cmudict.txt')
 CMU_DICT={}
-CACHE_DICT_FN=os.path.join(PATH_LANG_DATA,'tts-cache2.txt')
+CACHE_DICT_FN=os.path.join(PATH_LANG_DATA,'tts-cache.txt')
 CACHE_DICT_F=None
 CACHE=defaultdict(list)
 ORTH_CACHE=defaultdict(list)
@@ -45,8 +47,6 @@ def get_special_cases():
 The only necessary functions
 """
 
-
-
 def scan(line,incl_alt=True,**y):
     o=[
         {
@@ -62,61 +62,41 @@ def tokenize(txt,**y):
     # return tokenize_agnostic(txt,**y)
     return tokenize_nice(txt,**y)
 
+def get_df(token,**kwargs):
+    return pd.DataFrame(get(token,**kwargs))
 
-# def tokenize(txt):
-# 	import nltk
-# 	l=nltk.word_tokenize(txt,language="english")
-# 	l2=[]
-# 	for w in l:
-# 		if l2 and (w.startswith("'") or w.lower() in CONTRACTIONS):
-# 			l2[-1]+=w
-# 		else:
-# 			l2+=[w]
-# 	return l2
-
-
-def get(token,config={},toprint=False,incl_alt=True,cache_new=True):
+def get(token,config={},toprint=False,incl_alt=True,cache_new=True,**kwargs):
     # not real?
     token_nice=token
-    token = zero_punc(token).lower()
-    if not token:# or not token_alpha:
-        return [{
-            'word_ipa_i':0,
-            'syll_i':0,
-            'word_str':token,
-            'word_ipa':'',
-            'syll_ipa':'',
-            'syll_str':token,
-        }]
-
-
+    # token = zero_punc(token).lower()
+    if not token: return []
     # get ipas
     cache = get_cache()
-#     tokenl=token.lower()
+    # tokenl=token.lower()
+    tokenl=to_token(token)
     if token in cache:
         ipas=cache[token]
-#     elif tokenl in cache:
-#         ipas=cache[tokenl]
+    elif tokenl in cache:
+        ipas=cache[tokenl]
     else:
-        ipas=tts(token)
+        ipas=tts(tokenl)
         if ipas:
-            cache[token]=ipas
+            cache[tokenl]=ipas
             if cache_new:
-                write_to_cache(token,ipas[0])
-
+                write_to_cache(tokenl,ipas[0])
     ipas = ipas[:1] if not incl_alt else ipas
     sd=get_special_cases()
-    if token in sd['maybestressed']:
+    if tokenl in sd['maybestressed']:
         if len(ipas)<2:
             ipa=ipas[0]
             ipax="'"+ipa if ipa[0].isalpha() else ipa[1:]
             ipas.append(ipax)
-    elif token in sd['unstressed']:
+    elif tokenl in sd['unstressed']:
         ipa=ipas[0]
         ipax=ipa if ipa[0].isalpha() else ipa[1:]
         ipas=[ipax]
 
-    is_funcword=token in sd['functionwords']
+    is_funcword=tokenl in sd['functionwords']
 
     # get orths
     results = set()
@@ -134,15 +114,18 @@ def get(token,config={},toprint=False,incl_alt=True,cache_new=True):
         is_funcword_now = int(is_funcword and num_sylls==1 and ipa[0]!="'")
 
         for si,(syll_ipa,syll_text) in enumerate(zip(sylls_ipa, sylls_text)):
-            results_ld.append({
+            rdx={
                 'word_ipa_i':ipa_i+1,
                 'syll_i':si+1,
-                'word_str':token_nice,
+                # 'word_str':token_nice,
+                'word_tok':tokenl,
                 'word_ipa':ipa,
+                'word_nsyll':num_sylls,
                 'syll_ipa':syll_ipa,
                 'syll_str':syll_text,
-                'is_funcword':is_funcword_now
-            })
+                'word_isfunc':is_funcword_now
+            }
+            results_ld.append(rdx)
     return results_ld
 
 
@@ -166,47 +149,18 @@ def get_cache(source_paths=[CMU_DICT_FN,CACHE_DICT_FN]):
 
 def tts(token):
     espeak_ipa=espeak2ipa(token)
+    if espeak_ipa is None: return []
     cmu=espeak2cmu(espeak_ipa)
     cmu_sylls = syllabify_cmu(cmu)
     ipa = cmusylls2ipa(cmu_sylls)
     return [ipa] if ipa else []
 
 
-# loading dicts
-def load_cmu(fn=CMU_DICT_FN,config={}):
-    global CMU_DICT
-    fns=[fn]
-    if config.get('en_TTS_cache',False):
-        fns+=[CACHE_DICT_FN]
 
-    for fn in fns:
-        #print '>> loading words from:',fn
-        if os.path.exists(fn):
-            with open(fn,encoding='utf-8') as f:
-                for ln in f:
-                    ln=ln.strip()
-                    if not ln or ln.count('\t')!=1: continue
-                    word,ipa=ln.split('\t')[:2]
-                    word=word.strip()
-                    ipa=ipa.strip().split()[0]
-                    if not word in CMU_DICT: CMU_DICT[word]=[]
-                    CMU_DICT[word]+=[ipa]
-    return CMU_DICT
 
 def write_to_cache(token,ipa):
     with open(CACHE_DICT_FN,'a+',encoding='utf-8') as of:
         of.write(f'{token}\t{ipa}\n')
-    #pass
-#     tokenl=token.lower()
-#     global CACHE_DICT_F
-#     if not CACHE_DICT_F:
-#         CACHE_DICT_F=open(CACHE_DICT_FN,'a+',encoding='utf-8')
-
-#     CACHE_DICT_F.write(tokenl+'\t'+ipa+'\n')
-#     if not tokenl in CMU_DICT:
-#         CMU_DICT[tokenl]=[]
-#     CMU_DICT[tokenl]+=[ipa]
-#     return CMU_DICT
 
 
 def add_elisions(_ipa):
@@ -289,25 +243,40 @@ def add_elisions(_ipa):
 
 
 
+import shutil
+ERROR_SHOWN=False
+ERROR_MSG="""
+[cadence] A word unknown to the pronunciation dictionary ("{{WORD}}") cannot be syllabified because espeak text-to-speech software is not installed. It is highly recommend you install this software.
+
+To install:
+
+    * On Linux, type into the terminal:
+        apt-get install espeak
+    
+    * On Mac:
+        (1) Install homebrew if not already installed. Paste into Terminal app:
+            /bin/bash -c "$(curl -fsSL https://raw.githubusercontent.com/Homebrew/install/HEAD/install.sh)"
+
+        (2) Type into the Terminal app:
+            brew install espeak
+    
+    * On Windows:
+        Download and install from http://espeak.sourceforge.net/download.html.
+"""
 
 def espeak2ipa(token):
+    global ERROR_SHOWN
+    if not shutil.which('espeak'):
+        eprint(ERROR_MSG.replace('{{WORD}}',token))
+        ERROR_SHOWN=True
+        return None
+    
     token=''.join(x for x in token if x.isalpha())
     CMD=f'espeak -q -x {token}'
-#     print(CMD)
-    #CMD='espeak --ipa -q -x '+token.replace("'","\\'").replace('"','\\"')
-    #print CMD
     try:
-        # @HACK FOR MPI
-        #for k in os.environ.keys():
-        #	if k.startswith('OMPI_') or k.startswith('PMIX_'):
-        #		del os.environ[k]
-        ##
-
         res=subprocess.check_output(CMD.split()).strip()
-        #print '>> espeak = ',[res]
         return res.decode("utf-8")
     except (OSError,subprocess.CalledProcessError) as e:
-        #print "!!",e
         return None
 
 def tts2ipa(token,TTS_ENGINE=None):
